@@ -11,6 +11,8 @@ use serde::{Deserialize, Serialize};
 use time::{format_description::well_known::Rfc3339, OffsetDateTime};
 use uuid::Uuid;
 
+use crate::cmd::RemoveOrModifyNodeType;
+
 const SCHEMA_VERSION: &str = "0.0.1";
 const JETTY_VERSION: &str = env!("CARGO_PKG_VERSION");
 const FIRESTORE_URL: &str = "https://firestore.googleapis.com/v1/projects/jetty-cli-telemetry/databases/(default)/documents/";
@@ -163,9 +165,9 @@ pub enum UsageEvent {
     /// No args
     #[serde(rename = "invoked_default")]
     InvokedDefault,
-    /// `jetty init`
-    #[serde(rename = "invoked_init")]
-    InvokedInit,
+    /// `jetty new`
+    #[serde(rename = "invoked_new")]
+    InvokedNew,
     /// `jetty add`
     #[serde(rename = "invoked_add")]
     InvokedAdd,
@@ -180,7 +182,31 @@ pub enum UsageEvent {
     InvokedHelp,
     /// Program panicked during execution.
     #[serde(rename = "panicked")]
-    InvokedPanic,
+    Panic,
+    /// `jetty bootstrap`
+    #[serde(rename = "invoked_bootstrap")]
+    InvokedBootstrap { no_fetch: bool, overwrite: bool },
+    /// `jetty diff`
+    #[serde(rename = "invoked_diff")]
+    InvokedDiff { fetch: bool },
+    /// `jetty plan`
+    #[serde(rename = "invoked_plan")]
+    InvokedPlan { fetch: bool },
+    /// `jetty apply`
+    #[serde(rename = "invoked_apply")]
+    InvokedApply { no_fetch: bool },
+    /// `jetty subgraph`
+    #[serde(rename = "invoked_subgraph")]
+    InvokedSubgraph { depth: usize },
+    /// `jetty remove`
+    #[serde(rename = "invoked_remove")]
+    InvokedRemove { node_type: RemoveOrModifyNodeType },
+    /// `jetty rename`
+    #[serde(rename = "invoked_rename")]
+    InvokedRename { node_type: RemoveOrModifyNodeType },
+    /// `jetty rename`
+    #[serde(rename = "invoked_dev")]
+    InvokedDev,
 }
 
 /// Given an event, record its usage to Jetty anonymous usage stats.
