@@ -10,7 +10,6 @@ use crate::cual::cual_from_snowflake_obj_name;
 #[derive(Deserialize, Serialize, Clone, PartialEq, Eq, Hash)]
 pub enum GrantType {
     Standard(StandardGrant),
-    Future(super::future_grant::FutureGrant),
 }
 
 pub trait Grant {
@@ -26,39 +25,35 @@ pub trait Grant {
     }
 }
 
+/// This can be totally reworked, but just leaving it is as
 impl Grant for GrantType {
     fn granted_on_name(&self) -> &str {
         match self {
             GrantType::Standard(s) => s.granted_on_name(),
-            GrantType::Future(f) => f.granted_on_name(),
         }
     }
 
     fn role_name(&self) -> &str {
         match self {
             GrantType::Standard(s) => s.role_name(),
-            GrantType::Future(f) => f.role_name(),
         }
     }
 
     fn privilege(&self) -> &str {
         match self {
             GrantType::Standard(s) => s.privilege(),
-            GrantType::Future(f) => f.privilege(),
         }
     }
 
     fn granted_on(&self) -> &str {
         match self {
             GrantType::Standard(s) => s.granted_on(),
-            GrantType::Future(f) => f.granted_on(),
         }
     }
 
     fn into_policy(self, all_privileges: HashSet<String>) -> nodes::RawPolicy {
         match self {
             GrantType::Standard(s) => s.into_policy(all_privileges),
-            GrantType::Future(f) => f.into_policy(all_privileges),
         }
     }
 }
@@ -93,7 +88,7 @@ impl Grant for StandardGrant {
     }
 
     fn into_policy(self, all_privileges: HashSet<String>) -> nodes::RawPolicy {
-        let cual = cual_from_snowflake_obj_name(self.granted_on_name()).unwrap();
+        let cual = cual_from_snowflake_obj_name(self.granted_on_name(), self.granted_on()).unwrap();
 
         nodes::RawPolicy::new(
             format!("snowflake.{}.{}", self.role_name(), self.granted_on_name()),
@@ -146,7 +141,7 @@ mod tests {
             nodes::RawPolicy::new(
                 "snowflake.grantee_name.db".to_owned(),
                 HashSet::from(["priv".to_owned()]),
-                HashSet::from([cual_from_snowflake_obj_name("DB")?.uri()]),
+                HashSet::from([cual_from_snowflake_obj_name("DB", "DATABASE")?.uri()]),
                 HashSet::new(),
                 HashSet::from(["grantee_name".to_owned()]),
                 HashSet::new(),
